const log = require("@ui5/logger").getLogger("builder:customtask:typescript");
const ts = require('typescript');

const Compiler = require("ui5-typescript").Compiler;

class TypescriptTask {

    constructor(workspace, dependencies, options) {
        this.workspace = workspace;
        this.dependencies = dependencies;
        this.options = options;
    }


    getTsConfigFileName() {
        return (this.options.tsConfigFile) ? this.options.tsConfigFile : "./tsconfig.json"
    }

    run() {

        let compiler = new Compiler(this.getTsConfigFileName());
        let errorOccurred;

        let format = {
            getCanonicalFileName: path => path,
            getCurrentDirectory: ts.sys.getCurrentDirectory,
            getNewLine: () => ts.sys.newLine
        };
        
        return this.workspace.byGlob("/**/*.ts").then(resources => {
            return Promise.all(resources.map(tsResource => {
                compiler.compileResource(tsResource).then(result => {
            
                    result.diagnostics.forEach(diagnostic => {
                        errorOccurred = true;
                        if (diagnostic.file) {
                            if(diagnostic.file.fileName === tsResource.getPath()) {
                                log.error(ts.formatDiagnosticsWithColorAndContext([diagnostic], format));
                            }
                        } else {
                            format.getNewLine = () => "",
                            log.error(ts.formatDiagnosticsWithColorAndContext([diagnostic], format));
                        }
                    });

                    if(!result.diagnostics.length) {
                        this.workspace.write(result.jsResource);
                    }

                });
            })).then(() => {
                if (errorOccurred)
                    throw Error("Failed to compile TypeScript!");
            });  
        });
    }
    
}

/**
 * @param {Object} parameters Parameters
 * @param {DuplexCollection} parameters.workspace DuplexCollection to read and write files
 * @param {AbstractReader} parameters.dependencies Reader or Collection to read dependency files
 * @param {Object} parameters.options Options
 * @param {string} parameters.options.projectName Project name
 * @param {string} [parameters.options.configuration] Task configuration if given in ui5.yaml
 * @returns {Promise<undefined>} Promise resolving with undefined once data has been written
 */
module.exports = function({workspace, dependencies, options}) {
    return new TypescriptTask(workspace, dependencies, options).run();
};